@@ -8,6 +8,7 @@ each manages its own receivers and message emission.
 The :func:`signal` function provides singleton behavior for named signals.
 
 """
+from warnings import warn
 from weakref import WeakValueDictionary
 
 from blinker._utilities import (
@@ -15,6 +16,7 @@ from blinker._utilities import (
     contextmanager,
     defaultdict,
     hashable_identity,
+    lazy_property,
     reference,
     symbol,
     )
@@ -32,6 +34,42 @@ class Signal(object):
     #: without an additional import.
     ANY = ANY
 
+    @lazy_property
+    def receiver_connected(self):
+        """Emitted after each :meth:`connect`.
+
+        The signal sender is the signal instance, and the :meth:`connect`
+        arguments are passed through: *receiver*, *sender*, and *weak*.
+
+        .. versionadded:: 1.2
+
+        """
+        return Signal(doc="Emitted after a receiver connects.")
+
+    @lazy_property
+    def receiver_disconnected(self):
+        """Emitted after :meth:`disconnect`.
+
+        The sender is the signal instance, and the :meth:`disconnect` arguments
+        are passed through: *receiver* and *sender*.
+
+        Note, this signal is emitted **only** when :meth:`disconnect` is
+        called explicitly.
+
+        The disconnect signal can not be emitted by an automatic disconnect
+        (due to a weakly referenced receiver or sender going out of scope),
+        as the receiver and/or sender instances are no longer available for
+        use at the time this signal would be emitted.
+
+        An alternative approach is available by subscribing to
+        :attr:`receiver_connected` and setting up a custom weakref cleanup
+        callback on weak receivers and senders.
+
+        .. versionadded:: 1.2
+
+        """
+        return Signal(doc="Emitted after a receiver disconnects.")
+
     def __init__(self, doc=None):
         """
         :param doc: optional.  If provided, will be assigned to the signal's
@@ -98,6 +136,16 @@ class Signal(object):
                 del sender_ref
 
         # broadcast this connection.  if receivers raise, disconnect.
+        if ('receiver_connected' in self.__dict__ and
+            self.receiver_connected.receivers):
+            try:
+                self.receiver_connected.send(self,
+                                             receiver=receiver,
+                                             sender=sender,
+                                             weak=weak)
+            except:
+                self.disconnect(receiver, sender)
+                raise
         if receiver_connected.receivers and self is not receiver_connected:
             try:
                 receiver_connected.send(self,
@@ -109,31 +157,58 @@ class Signal(object):
                 raise
         return receiver
 
+    def connect_via(self, sender, weak=False):
+        """Connect the decorated function as a receiver for *sender*.
+
+        :param sender: Any object or :obj:`ANY`.  The decorated function
+          will only receive :meth:`send` emissions sent by *sender*.  If
+          ``ANY``, the receiver will always be notified.  A function may be
+          decorated multiple times with differing *sender* values.
+
+        :param weak: If true, the Signal will hold a weakref to the
+          decorated function and automatically disconnect when *receiver*
+          goes out of scope or is garbage collected.  Unlike
+          :meth:`connect`, this defaults to False.
+
+        The decorated function will be invoked by :meth:`send` with
+          `sender=` as a single positional argument and any \*\*kwargs that
+          were provided to the call to :meth:`send`.
+
+
+        .. versionadded:: 1.1
+
+        """
+        def decorator(fn):
+            self.connect(fn, sender, weak)
+            return fn
+        return decorator
+
     @contextmanager
-    def temporarily_connected_to(self, receiver, sender=ANY):
-        """Execute a block with the signal connected *receiver*.
+    def connected_to(self, receiver, sender=ANY):
+        """Execute a block with the signal temporarily connected to *receiver*.
 
         :param receiver: a receiver callable
         :param sender: optional, a sender to filter on
 
-        This is a context manager for use in the ``with`` statement. It can
+        This is a context manager for use in the ``with`` statement.  It can
         be useful in unit tests.  *receiver* is connected to the signal for
         the duration of the ``with`` block, and will be disconnected
         automatically when exiting the block:
 
         .. testsetup::
 
+          from __future__ import with_statement
           from blinker import Signal
           on_ready = Signal()
           receiver = lambda sender: None
 
         .. testcode::
 
-          with on_ready.temporarily_connected_to(receiver):
+          with on_ready.connected_to(receiver):
              # do stuff
              on_ready.send(123)
 
-        .. versionadded:: 0.9
+        .. versionadded:: 1.1
 
         """
         self.connect(receiver, sender=sender, weak=False)
@@ -145,6 +220,24 @@ class Signal(object):
         else:
             self.disconnect(receiver)
 
+    def temporarily_connected_to(self, receiver, sender=ANY):
+        """An alias for :meth:`connected_to`.
+
+        :param receiver: a receiver callable
+        :param sender: optional, a sender to filter on
+
+        .. versionadded:: 0.9
+
+        .. versionchanged:: 1.1
+          Renamed to :meth:`connected_to`.  ``temporarily_connected_to``
+          was deprecated in 1.2 and removed in a subsequent version.
+
+        """
+        warn("temporarily_connected_to is deprecated; "
+             "use connected_to instead.",
+             DeprecationWarning)
+        return self.connected_to(receiver, sender)
+
     def send(self, *sender, **kwargs):
         """Emit this signal on behalf of *sender*, passing on \*\*kwargs.
 
@@ -227,6 +320,12 @@ class Signal(object):
         receiver_id = hashable_identity(receiver)
         self._disconnect(receiver_id, sender_id)
 
+        if ('receiver_disconnected' in self.__dict__ and
+            self.receiver_disconnected.receivers):
+            self.receiver_disconnected.send(self,
+                                            receiver=receiver,
+                                            sender=sender)
+
     def _disconnect(self, receiver_id, sender_id):
         if sender_id == ANY_ID:
             if self._by_receiver.pop(receiver_id, False):
@@ -264,6 +363,13 @@ Sent by a :class:`Signal` after a receiver connects.
 :keyword sender_arg: the sender to connect to
 :keyword weak_arg: true if the connection to receiver_arg is a weak reference
 
+.. deprecated:: 1.2
+
+As of 1.2, individual signals have their own private
+:attr:`~Signal.receiver_connected` and
+:attr:`~Signal.receiver_disconnected` signals with a slightly simplified
+call signature.  This global signal is planned to be removed in 1.6.
+
 """)
 
 
@@ -281,7 +387,7 @@ class NamedSignal(Signal):
         return "%s; %r>" % (base[:-1], self.name)
 
 
-class Namespace(WeakValueDictionary):
+class Namespace(dict):
     """A mapping of signal names to signals."""
 
     def signal(self, name, doc=None):
@@ -296,4 +402,25 @@ class Namespace(WeakValueDictionary):
             return self.setdefault(name, NamedSignal(name, doc))
 
 
+class WeakNamespace(WeakValueDictionary):
+    """A weak mapping of signal names to signals.
+
+    Automatically cleans up unused Signals when the last reference goes out
+    of scope.  This namespace implementation exists for a measure of legacy
+    compatibility with Blinker <= 1.2, and may be dropped in the future.
+
+    """
+
+    def signal(self, name, doc=None):
+        """Return the :class:`NamedSignal` *name*, creating it if required.
+
+        Repeated calls to this function will return the same signal object.
+
+        """
+        try:
+            return self[name]
+        except KeyError:
+            return self.setdefault(name, NamedSignal(name, doc))
+
+
 signal = Namespace().signal
